import { EMPTY, of, ReplaySubject, Subscription } from 'rxjs';
import { catchError, distinctUntilChanged, filter, map, switchMap, tap } from 'rxjs/operators';
import { nameToStrategy } from './nameToStrategy';
/**
 * RenderAware
 *
 * @description
 * This function returns an object that holds all the shared logic for the push pipe and the let directive
 * responsible for change detection
 * If you extend this class you need to implement how the update of the rendered value happens.
 * Also custom behaviour is something you need to implement in the extending class
 */
export function createRenderAware(cfg) {
    const strategyName$ = new ReplaySubject(1);
    let currentStrategy;
    const strategy$ = strategyName$.pipe(distinctUntilChanged(), switchMap(stringOrObservable => typeof stringOrObservable === 'string'
        ? of(stringOrObservable)
        : stringOrObservable), nameToStrategy(cfg.strategies), tap(s => (currentStrategy = s)));
    const observablesFromTemplate$ = new ReplaySubject(1);
    const valuesFromTemplate$ = observablesFromTemplate$.pipe(distinctUntilChanged());
    let firstTemplateObservableChange = true;
    const renderingEffect$ = valuesFromTemplate$.pipe(
    // handle null | undefined assignment and new Observable reset
    map(observable$ => {
        if (observable$ === null) {
            return of(null);
        }
        if (!firstTemplateObservableChange) {
            cfg.resetObserver.next();
            if (observable$ === undefined) {
                return of(undefined);
            }
        }
        firstTemplateObservableChange = false;
        return observable$;
    }), 
    // forward only observable values
    filter(o$ => o$ !== undefined), switchMap(o$ => o$.pipe(distinctUntilChanged(), tap(cfg.updateObserver), currentStrategy.rxScheduleCD, tap({
        // handle "error" and "complete" notifications for Observable from template
        error: err => {
            console.error(err);
            if (cfg.updateObserver.error) {
                cfg.updateObserver.error(err);
                currentStrategy.detectChanges();
            }
        },
        complete: cfg.updateObserver.complete
            ? () => currentStrategy.detectChanges()
            : undefined
    }))), catchError(e => {
        return EMPTY;
    }));
    return {
        nextPotentialObservable(value) {
            observablesFromTemplate$.next(value);
        },
        nextStrategy(nextConfig) {
            strategyName$.next(nextConfig);
        },
        activeStrategy$: strategy$,
        subscribe() {
            return new Subscription()
                .add(strategy$.subscribe())
                .add(renderingEffect$.subscribe());
        }
    };
}
//# sourceMappingURL=render-aware_creator.js.map