import { AfterViewInit, Directive, ElementRef, Input, OnDestroy } from '@angular/core';
import { BehaviorSubject, Subscription } from 'rxjs';
import { tap } from 'rxjs/operators';
import { zonePatchedEvents } from './unpatch-event-list';
import { getZoneUnPatchedApi } from '../../../core/utils';
/**
 *
 * @description
 *
 * This function takes an elem and event and re-applies the listeners from the passed event to the
 * passed element with the zone un-patched version of it.
 *
 * @param elem {HTMLElement} - The elem to re-apply the listeners to.
 * @param event {string} - The name of the event from which to re-apply the listeners.
 *
 * @returns void
 */
export function unpatchEventListener(elem, event) {
    var eventListeners = elem.eventListeners(event);
    // Return if no event listeners are present
    if (!eventListeners) {
        return;
    }
    var addEventListener = getZoneUnPatchedApi('addEventListener', elem).bind(elem);
    eventListeners.forEach(function (listener) {
        // Remove and reapply listeners with patched API
        elem.removeEventListener(event, listener);
        // Reapply listeners with un-patched API
        addEventListener(event, listener);
    });
}
/**
 * @Directive UnpatchEventsDirective
 *
 * @description
 *
 * The `unpatch` directive helps in partially migrating to zone-less apps as well as getting rid
 * of unnecessary renderings through zones `addEventListener` patches.
 * It can be used on any element you apply event bindings.
 *
 * The current way of binding events to the DOM is to use output bindings:
 *  ```html
 * <button (click)="doStuff($event)">click me</button>
 * ```
 *
 * The problem is that every event registered over `()` syntax, e.g. `(click)`
 * marks the component and all its ancestors as dirty and re-renders the whole component tree.
 * This is because zone.js patches the native browser API and whenever one of the patched APIs is used it re-renders.
 *
 * So even if your button is not related to a change that needs a re-render the app will re-render completely.
 * This leads to bad performance. This is especially helpful if you work with frequently fired events like 'mousemove'
 *
 * `unpatch` directive solves that problem.
 *
 * Included Features:
 *  - by default un-patch all registered listeners of the host it is applied on
 *  - un-patch only a specified set of registered event listeners
 *  - works zone independent (it directly checks the widow for patched APIs and un-patches them without the use of `runOutsideZone` which brings more performance)
 *  - Not interfering with any logic executed by the registered callback
 *
 * @usageNotes
 *
 * The `unpatch` directive can be used like shown here:
 * ```html
 * <button [unoatch] (click)="triggerSomeMethod($event)">click me</button>
 * <button [unoatch]="['mousemove']" (mousemove)="doStuff2($event)" (click)="doStuff($event)">click me</button>
 * ```
 *
 * @publicApi
 */
// tslint:disable-next-line:directive-selector
var UnpatchEventsDirective = /** @class */ (function () {
    function UnpatchEventsDirective(el) {
        this.el = el;
        this.subscription = new Subscription();
        this.events$ = new BehaviorSubject(zonePatchedEvents);
    }
    Object.defineProperty(UnpatchEventsDirective.prototype, "events", {
        set: function (value) {
            if (value && value.length > 0) {
                this.events$.next(value);
            }
            else {
                this.events$.next(zonePatchedEvents);
            }
        },
        enumerable: false,
        configurable: true
    });
    UnpatchEventsDirective.prototype.reapplyEventListenersZoneUnPatched = function (events) {
        var _this = this;
        events.forEach(function (ev) {
            unpatchEventListener(_this.el.nativeElement, ev);
        });
    };
    UnpatchEventsDirective.prototype.ngOnDestroy = function () {
        this.subscription.unsubscribe();
    };
    UnpatchEventsDirective.prototype.ngAfterViewInit = function () {
        var _this = this;
        this.subscription = this.events$
            .pipe(tap(function (eventList) { return _this.reapplyEventListenersZoneUnPatched(eventList); }))
            .subscribe();
    };
    UnpatchEventsDirective.ctorParameters = function () { return [
        { type: ElementRef }
    ]; };
    UnpatchEventsDirective.decorators = [
        { type: Directive, args: [{ selector: '[unpatch]' },] }
    ];
    UnpatchEventsDirective.ctorParameters = function () { return [
        { type: ElementRef }
    ]; };
    UnpatchEventsDirective.propDecorators = {
        events: [{ type: Input, args: ['unpatch',] }]
    };
    return UnpatchEventsDirective;
}());
export { UnpatchEventsDirective };
//# sourceMappingURL=unpatch-events.directive.js.map